"""djLint Attempt to logically shrink html."""

import regex as re

from ..settings import Config


def _clean_line(line: str) -> str:
    """Clean up a line of html.

    * remove duplicate spaces
    * remove trailing spaces

    """
    return re.sub(r" {2,}", " ", line.strip())


def _strip_html_whitespace(html: str, config: Config) -> str:
    """Remove unnecessary whitespace from text."""
    rawcode_flat = ""
    is_block_ignored = False
    is_group_ignored = False

    for item in html.strip().splitlines():

        # start of ignored block. If we are already in an ingored block, keep true.
        is_group_ignored = is_group_ignored or bool(
            re.search(config.ignored_group_opening, item, re.IGNORECASE | re.VERBOSE)
        )

        # find ignored blocks and retain indentation, otherwise strip white space
        if re.findall(
            r"(?:%s)" % config.ignored_inline_blocks,
            item,
            flags=re.IGNORECASE | re.VERBOSE,
        ):
            tmp = _clean_line(item)

        elif (
            re.search(config.ignored_block_closing, item, re.IGNORECASE | re.VERBOSE)
            and is_group_ignored is False
        ):
            # do not format ignored lines
            tmp = _clean_line(item)
            is_block_ignored = False

        elif (
            re.search(config.ignored_block_opening, item, re.IGNORECASE | re.VERBOSE)
            and is_group_ignored is False
        ):
            # do not format ignored lines
            tmp = _clean_line(item)
            is_block_ignored = True

        elif is_group_ignored or is_block_ignored:
            tmp = item

        else:
            tmp = _clean_line(item)

        # end of ignore raw code
        if bool(
            re.search(config.ignored_group_closing, item, re.IGNORECASE | re.VERBOSE)
        ):
            is_group_ignored = False

        rawcode_flat = rawcode_flat + tmp + "\n"

    return rawcode_flat


def compress_html(html: str, config: Config) -> str:
    """Compress back tags that do not need to be expanded."""
    # put empty tags on one line
    html = _strip_html_whitespace(html, config)

    html = re.sub(
        r"(<([\w]+)[^>]*>)\s+?(<\/\2>)",
        r"\1\3",
        html,
        flags=re.IGNORECASE | re.MULTILINE,
    )

    # put empty template tags on one line
    html = re.sub(
        r"({%[ ]*?("
        + re.sub(r"\s", "", config.start_template_tags)
        + r")[^}]+?%})\s+?(\{\% end[^}]*?\%\})",
        r"\1\3",
        html,
        re.MULTILINE,
    )

    # put short single line tags on one line
    # verbose doesn't seem to work with replace groups.
    slt_html = re.sub(r"\s", "", config.single_line_html_tags)

    html = re.sub(
        r"(<(%s)>)\s*([^<\n]{,80})\s*?(</(\2)>)" % slt_html,
        r"\1\3\4",
        html,
        re.IGNORECASE | re.MULTILINE | re.DOTALL,
    )

    html = re.sub(
        r"(<(%s)>)\s*?([^<\n]{,80})\s*?(</(\2)>)" % slt_html,
        r"\1\3\4",
        html,
        re.IGNORECASE | re.MULTILINE | re.DOTALL,
    )

    html = re.sub(
        r"(<(%s)[ ][^\n]{,80}>)\s*([^<\n]{,80})\s*?(</(\2)>)" % slt_html,
        r"\1\3\4",
        html,
        re.IGNORECASE | re.MULTILINE | re.DOTALL,
    )

    # cannot use verbose when replacing with var. 🤕
    html = re.sub(
        r"({%[ ]*?("
        + re.sub(r"\s", "", config.single_line_template_tags)
        + r")[ ]+?[^\n]{,30}%})\s*([^%\n]{,50})\s*?({%[ ]+?end(\2)[ ]*?%})",
        r"\1\3\4",
        html,
        re.IGNORECASE | re.MULTILINE,
    )

    return html



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['djlint', 'djlint.formatter']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'click>=8.0.1,<9.0.0',
 'colorama>=0.4.4,<0.5.0',
 'regex>=2021.8.28,<2022.0.0',
 'tomlkit>=0.7.2,<0.8.0',
 'tqdm>=4.62.2,<5.0.0']

extras_require = \
{'test': ['coverage>=5.5,<6.0',
          'pytest>=6.2.5,<7.0.0',
          'pytest-cov>=2.12.1,<3.0.0']}

entry_points = \
{'console_scripts': ['djlint = djlint:main']}

setup_kwargs = {
    'name': 'djlint',
    'version': '0.3.5',
    'description': 'HTML Template Linter and Formatter',
    'long_description': '![djLint Logo](https://raw.githubusercontent.com/Riverside-Healthcare/djlint/master/docs/_static/icon.png)\n\nFind common formatting issues and *reformat* HTML templates.\n\n<style>\n      .types {\n        text-align: center;\n        font-size: 110%;\n        font-style: italic;\n        width: 100%;\n        position: relative;\n      }\n\n      .types::after {\n        position: absolute;\n        top: 11px;\n        left:0;\n        right:0;\n        border: 1px solid rgba(0,0,0,.2);\n        content: "";\n        z-index: -1;\n      }\n\n      .types span {\n        padding: 15px;\n        margin: 5px;\n        background: #fff;\n        z-index:1;\n      }\n   </style>\n   <p class="types">\n      <span>Django</span>\n      <span>Jinja</span>\n      <span>Nunjucks</span>\n      <span>Handlebars</span>\n   </p>\n\nPs, ```--check``` it out on Jinja and Handlebar templates as well!\n\n[![codecov](https://codecov.io/gh/Riverside-Healthcare/djlint/branch/master/graph/badge.svg?token=eNTG721BAA)](https://codecov.io/gh/Riverside-Healthcare/djlint) [![test](https://github.com/Riverside-Healthcare/djlint/actions/workflows/test.yml/badge.svg)](https://github.com/Riverside-Healthcare/djlint/actions/workflows/test.yml) [![Codacy Badge](https://app.codacy.com/project/badge/Grade/dba6338b0e7a4de896b45b382574f369)](https://www.codacy.com/gh/Riverside-Healthcare/djlint/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=Riverside-Healthcare/djlint&amp;utm_campaign=Badge_Grade) [![Maintainability](https://api.codeclimate.com/v1/badges/5febe4111a36c7e0d2ed/maintainability)](https://codeclimate.com/github/Riverside-Healthcare/djlint/maintainability) [![Downloads](https://pepy.tech/badge/djlint)](https://pepy.tech/project/djlint)[![chat](https://img.shields.io/badge/chat-discord-green)](https://discord.gg/taghAqebzU)\n\n## Documentation\n\nRead the [documentation](https://djlint.readthedocs.io)\n\n## Installation and Usage\n\n**djLint** can be installed with `pip install djlint`, and is easy to run:\n\n```sh\n# to lint a directory\ndjlint /path\n\n# to lint a directory with custom extension\ndjlint /path -e html.dj\n\n# to check formatting on a file\ndjlint /path/file.html.j2 --check\n\n# to reformt a directory without printing the file diff\ndjlint /path --reformat --quiet\n\n```\n\n## Show your format\n\nAdd a badge to your projects ```readme.md```:\n\n```md\n[![Code style: djlint](https://img.shields.io/badge/html%20style-djlint-blue.svg)](https://github.com/Riverside-Healthcare/djlint)\n```\n\nAdd a badge to your ```readme.rst```:\n\n```rst\n.. image:: https://img.shields.io/badge/html%20style-djlint-blue.svg\n   :target: https://github.com/Riverside-Healthcare/djlint\n```\nLooks like this:\n\n[![djlint](https://img.shields.io/badge/html%20style-djlint-blue.svg)](https://github.com/Riverside-Healthcare/djlint)\n\n\n## Contributing\n\nCheckout the [issue](https://github.com/Riverside-Healthcare/djlint/issues) list and help where you can.\n',
    'author': 'Christopher Pickering',
    'author_email': 'cpickering@rhc.net',
    'maintainer': 'Christopher Pickering',
    'maintainer_email': 'cpickering@rhc.net',
    'url': 'https://github.com/Riverside-Healthcare/djlint',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""djLint add indentation to html."""

from functools import partial

import regex as re

from ..settings import Config


def _format_attributes(config: Config, match: re.match) -> str:
    """Spread long attributes over multiple lines."""
    leading_space = match.group(1)

    tag = match.group(2)

    spacing = "\n" + leading_space + len(tag) * " "

    attributes = (spacing).join(
        re.findall(str(config.attribute_pattern), match.group(3).strip(), re.VERBOSE)
    )

    close = match.group(4)

    return "{}{}{}{}".format(
        leading_space,
        tag,
        attributes,
        close,
    )


def indent_html(rawcode: str, config: Config) -> str:
    """Indent raw code."""
    rawcode_flat_list = re.split("\n", rawcode)

    indent = config.indent

    beautified_code = ""
    indent_level = 0
    is_block_raw = False

    slt_html = config.break_html_tags

    # here using all tags cause we allow empty tags on one line
    always_slt_html = config.always_single_line_html_tags

    # here using all tags cause we allow empty tags on one line
    slt_template = config.single_line_template_tags

    for item in rawcode_flat_list:
        # if a raw tag then start ignoring
        if re.search(config.ignored_group_opening, item, re.IGNORECASE | re.VERBOSE):
            is_block_raw = True

        if re.findall(
            config.ignored_inline_blocks, item, flags=re.IGNORECASE | re.VERBOSE
        ):
            tmp = (indent * indent_level) + item + "\n"

        # if a one-line, inline tag, just process it, only if line starts w/ it
        elif (
            re.findall(
                r"(<(%s)>)(.*?)(</(\2)>)" % slt_html, item, re.IGNORECASE | re.VERBOSE
            )
            or re.findall(
                r"(<(%s)[ ].+?>)(.*?)(</(\2)>)" % slt_html,
                item,
                re.IGNORECASE | re.VERBOSE,
            )
            or re.findall(
                r"^({%[ ]*?("
                + str(slt_template)
                + r")[ ]+?.+?%})(.*?)({%[ ]+?end(\2)[ ]+?.*?%})",
                item,
                re.IGNORECASE | re.MULTILINE | re.VERBOSE,
            )
            or re.findall(
                r"(<(%s)[ ].*?/>)" % slt_html, item, flags=re.IGNORECASE | re.VERBOSE
            )
            or re.findall(
                r"(<(%s)[ ].*?/?>)" % always_slt_html,
                item,
                flags=re.IGNORECASE | re.VERBOSE,
            )
        ) and is_block_raw is False:
            tmp = (indent * indent_level) + item + "\n"

        # if unindent, move left
        elif (
            re.search(
                config.tag_unindent,
                item,
                re.IGNORECASE | re.MULTILINE | re.VERBOSE,
            )
            and is_block_raw is False
            or re.search(config.ignored_block_closing, item, re.IGNORECASE | re.VERBOSE)
        ):
            indent_level = max(indent_level - 1, 0)
            tmp = (indent * indent_level) + item + "\n"

        elif (
            re.search(
                r"^" + str(config.tag_unindent_line),
                item,
                re.IGNORECASE | re.MULTILINE | re.VERBOSE,
            )
            and is_block_raw is False
        ):
            tmp = (indent * (indent_level - 1)) + item + "\n"

        # if indent, move right
        elif (
            re.search(
                r"^(?:" + str(config.tag_indent) + r")",
                item,
                re.IGNORECASE | re.MULTILINE | re.VERBOSE,
            )
            and is_block_raw is False
        ):
            tmp = (indent * indent_level) + item + "\n"
            indent_level = indent_level + 1

        elif is_block_raw is True:
            tmp = item + "\n"

        # if just a blank line
        elif item.strip() == "":
            tmp = item.strip()

        # otherwise, just leave same level
        else:
            tmp = (indent * indent_level) + item + "\n"

        # we can try to fix template tags
        tmp = re.sub(r"({[{|%]\-?)(\w[^}].+?)([}|%]})", r"\1 \2\3", tmp)
        tmp = re.sub(r"({[{|%])([^}].+?[^(?:\ |\-)])([}|%]})", r"\1\2 \3", tmp)
        tmp = re.sub(r"({[{|%])([^}].+?[^ ])(\-[}|%]})", r"\1\2 \3", tmp)

        # handlebars templates
        tmp = re.sub(r"({{#(?:each|if).+?[^ ])(}})", r"\1 \2", tmp)

        # if a opening raw tag then start ignoring.. only if there is no closing tag
        # on the same line
        if (
            re.search(config.ignored_group_opening, item, re.IGNORECASE | re.VERBOSE)
        ) or re.search(config.ignored_block_opening, item, re.IGNORECASE | re.VERBOSE):
            is_block_raw = True

        # if a normal tag, we can try to expand attributes
        elif (
            config.format_long_attributes
            and is_block_raw is False
            and len(tmp) > int(config.max_line_length)
        ):
            # get leading space, and attributes
            func = partial(_format_attributes, config)
            tmp = re.sub(r"(\s*?)(<\w+\s)([^>]+?)(/?>)", func, tmp)

        # turn off raw block if we hit end - for one line raw blocks
        if (
            re.search(
                re.sub(r"\s", "", config.ignored_group_closing), item, re.IGNORECASE
            )
        ) or re.search(
            re.sub(r"\s", "", config.ignored_block_closing), item, re.IGNORECASE
        ):

            is_block_raw = False

        beautified_code = beautified_code + tmp

    return beautified_code.strip() + "\n"
